@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2008,2009 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2008-2011 -- leonerd@leonerd.org.uk
 
 package IO::Async;
 
@@ -12,11 +12,11 @@ use warnings;
 # It is provided simply to keep CPAN happy:
 #   cpan -i IO::Async
 
-our $VERSION = '0.29';
+our $VERSION = '0.42';
 
 =head1 NAME
 
-C<IO::Async> - perform asynchronous filehandle IO and other operations
+C<IO::Async> - Asynchronous event-driven programming
 
 =head1 SYNOPSIS
 
@@ -30,20 +30,18 @@ C<IO::Async> - perform asynchronous filehandle IO and other operations
     service  => 12345,
     socktype => 'stream',
 
-    on_connected => sub {
-       my ( $socket ) = @_;
-
-       my $stream = IO::Async::Stream->new(
-          handle => $socket,
+    on_stream => sub {
+       my ( $stream ) = @_;
 
+       $stream->configure(
           on_read => sub {
-             my ( $self, $buffref, $closed ) = @_;
+             my ( $self, $buffref, $eof ) = @_;
 
-             return 0 unless( $buffref =~ s/^(.*\n)// );
+             while( $buffref =~ s/^(.*\n)// ) {
+                print "Received a line $1";
+             }
 
-             print "Received a line $1";
-
-             return 1;
+             return 0;
           }
        );
 
@@ -52,7 +50,8 @@ C<IO::Async> - perform asynchronous filehandle IO and other operations
        $loop->add( $stream );
     },
 
-    ...
+    on_resolve_error => sub { die "Cannot resolve - $_[-1]\n"; },
+    on_connect_error => sub { die "Cannot connect - $_[0] failed $_[-1]\n"; },
  );
 
  $loop->loop_forever();
@@ -94,6 +93,12 @@ class automatically handles reading of incoming data into the incoming buffer,
 and writing of the outgoing buffer. Methods or callbacks are used to inform
 when new incoming data is available, or when the outgoing buffer is empty.
 
+While stream-based sockets can be handled using using C<IO::Async::Stream>,
+datagram or raw sockets do not provide a bytestream. For these, the
+L<IO::Async::Socket> class is another subclass of L<IO::Async::Handle> which
+maintains an outgoing packet queue, and informs of packet receipt using a
+callback or method.
+
 The L<IO::Async::Listener> class is another subclass of L<IO::Async::Handle>
 which facilitates the use of C<listen()>-mode sockets. When a new connection
 is available on the socket it will C<accept()> it and pass the new client
@@ -101,6 +106,9 @@ socket to its callback function.
 
 =head2 Timers
 
+An L<IO::Async::Timer::Absolute> object represents a timer that expires at a
+given absolute time in the future.
+
 An L<IO::Async::Timer::Countdown> object represents a count time timer, which
 will invoke a callback after a given delay. It can be stopped and restarted.
 
@@ -119,12 +127,13 @@ callback when the given signal is received by the process. Multiple objects
 watching the same signal can be used; they will all invoke in no particular
 order.
 
-=head2 Merge Points
+=head2 Processes Management
 
-The L<IO::Async::MergePoint> object class allows for a program to wait on the
-completion of multiple seperate subtasks. It allows for each subtask to return
-some data, which will be collected and given to the callback provided to the
-merge point, which is called when every subtask has completed.
+An L<IO::Async::PID> object invokes its event when a given child process
+exits. An L<IO::Async::Process> object can start a new child process running
+either a given block of code, or executing a given command, set up pipes on
+its filehandles, write to or read from these pipes, and invoke its event when
+the child process exits.
 
 =head2 Loops
 
@@ -151,12 +160,11 @@ implementation exists for the specific OS it runs on.
 
 The L<IO::Async::Loop> object provides a number of methods to facilitate the
 running of child processes. C<spawn_child> is primarily a wrapper around the
-typical C<fork()>/C<exec()> style of starting child processes, C<open_child>
-builds on this to provide management of child process file handles and streams
-connected to them, and finally C<run_child> builds on that to provide a method
-similar to perl's C<readpipe()> (which is used to implement backticks C<``>).
+typical C<fork()>/C<exec()> style of starting child processes, and
+C<run_child> provide a method similar to perl's C<readpipe()> (which is used
+to implement backticks C<``>).
 
-=head2 Detached Code
+=head2 Asynchronous Functions
 
 The C<IO::Async> framework generally provides mechanisms for multiplexing IO
 tasks between different handles, so there aren't many occasions when it is
@@ -176,10 +184,10 @@ no asynchronous version is supplied.
 
 =back
 
-For these cases, an instance of L<IO::Async::DetachedCode> can be used around
-a code block, to execute it in a detached child process. The code in the
-sub-process runs isolated from the main program, communicating only by
-function call arguments and return values.
+For these cases, an instance of L<IO::Async::Function> can be used around
+a code block, to execute it in a worker child process or set of processes.
+The code in the sub-process runs isolated from the main program, communicating
+only by function call arguments and return values.
 
 =head2 Networking
 
@@ -187,7 +195,16 @@ The L<IO::Async::Loop> provides several methods for performing network-based
 tasks. Primarily, the C<connect> and C<listen> methods allow the creation of
 client or server network sockets. Additionally, the C<resolve> method allows
 the use of the system's name resolvers in an asynchronous way, to resolve
-names into addresses, or vice versa.
+names into addresses, or vice versa. These methods are fully IPv6-capable if
+the underlying operating system is.
+
+=head2 Protocols
+
+The L<IO::Async::Protocol> class provides storage for a L<IO::Async::Handle>
+object, to act as a transport for some protocol. It allows a level of
+independence from the actual transport being for that protocol, allowing it to
+be easily reused. The L<IO::Async::Protocol::Stream> subclass provides further
+support for protocols based on stream connections, such as TCP sockets.
 
 =head1 TODO
 
@@ -207,14 +224,14 @@ be useful on Win32.
 A consideration on how to provide per-OS versions of the utility classes. For
 example, Win32 would probably need an extensively-different C<ChildManager>,
 or OSes may have specific ways to perform asynchronous name resolution
-operations better than the generic C<DetachedCode> approach. This should be
-easier to implement now that the L<IO::Async::Loop> magic constructor looks
-for OS-specific subclasses first.
+operations better than the generic C<IO::Async::Function> approach. This
+should be easier to implement now that the L<IO::Async::Loop> magic
+constructor looks for OS-specific subclasses first.
 
 =item *
 
 A consideration of whether it is useful and possible to provide integration
-with L<POE> or L<AnyEvent>.
+with L<AnyEvent>.
 
 =back
 
@@ -226,17 +243,21 @@ with L<POE> or L<AnyEvent>.
 
 L<Event> - Event loop processing
 
-=item *
-
-L<POE> - portable multitasking and networking framework for Perl
-
 =back
 
+=head1 SUPPORT
+
+Bugs may be reported via RT at
+
+ https://rt.cpan.org/Public/Dist/Display.html?Name=IO-Async
+
+Support by IRC may also be found on F<irc.perl.org> in the F<#io-async>
+channel.
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
 
 =cut
 
-# Keep perl happy; keep Britain tidy
-1;
+0x55AA;
