@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2008,2009 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2008-2011 -- leonerd@leonerd.org.uk
 
 package IO::Async::Listener;
 
@@ -9,14 +9,13 @@ use strict;
 use warnings;
 use base qw( IO::Async::Handle );
 
-our $VERSION = '0.27';
+our $VERSION = '0.42';
 
 use IO::Async::Handle;
 
 use POSIX qw( EAGAIN );
-use Socket::GetAddrInfo qw( :Socket6api AI_PASSIVE );
 
-use Socket qw( SO_ACCEPTCONN SO_REUSEADDR );
+use Socket qw( sockaddr_family SOL_SOCKET SO_ACCEPTCONN SO_REUSEADDR SO_TYPE );
 
 use Carp;
 
@@ -27,25 +26,24 @@ C<IO::Async::Listener> - listen on network sockets for incoming connections
 =head1 SYNOPSIS
 
  use IO::Async::Listener;
- use IO::Async::Stream;
 
  use IO::Async::Loop;
  my $loop = IO::Async::Loop->new();
 
  my $listener = IO::Async::Listener->new(
-    on_accept => sub {
-       my ( $newclient ) = @_;
-
-       $loop->add( IO::Async::Stream->new(
-          handle => $newclient,
+    on_stream => sub {
+       my ( undef, $stream ) = @_;
 
+       $stream->configure(
           on_read => sub {
-             my ( $self, $buffref, $closed ) = @_;
+             my ( $self, $buffref, $eof ) = @_;
              $self->write( $$buffref );
              $$buffref = "";
              return 0;
           },
-       ) );
+       );
+       
+       $loop->add( $stream );
     },
  );
 
@@ -72,7 +70,7 @@ This object can also be used indirectly via an C<IO::Async::Loop>:
     service  => "echo",
     socktype => 'stream',
 
-    on_accept => sub {
+    on_stream => sub {
        ...
     },
 
@@ -92,27 +90,32 @@ Alternatively, the Listener can construct a socket by calling the C<listen>
 method. Either a list of addresses can be provided, or a service name can be
 looked up using the underlying loop's C<resolve> method.
 
-This object may be used in one of two ways; with a callback function, or as a
-base class.
+=cut
 
-=over 4
+=head1 EVENTS
 
-=item Callbacks
+The following events are invoked, either using subclass methods or CODE
+references in parameters:
 
-If the C<on_accept> key is supplied to the constructor, it should contain a
-CODE reference to a callback function to be invoked when a new client connects
-to the socket. It is passed an C<IO::Socket> reference to the newly accepted
-socket:
+=head2 on_accept $clientsocket
 
- $on_accept->( $self, $clientsocket )
+Invoked whenever a new client connects to the socket.
 
-=item Base Class
+=head2 on_stream $stream
 
-If a subclass is built, then it can override the C<on_accept> method.
+An alternative to C<on_accept>, this an instance of L<IO::Async::Stream> when
+a new client connects. This is provided as a convenience for the common case
+that a Stream object is required as the transport for a Protocol object.
 
- $self->on_accept( $clientsocket )
+=head2 on_socket $socket
 
-=back
+Similar to C<on_stream>, but constructs an instance of L<IO::Async::Socket>.
+This is most useful for C<SOCK_DGRAM> or C<SOCK_RAW> sockets.
+
+=head2 on_accept_error $socket, $errno
+
+Optional. Invoked if the C<accept> syscall indicates an error (other than
+C<EAGAIN>). If not provided, failures of C<accept> will simply be ignored.
 
 =cut
 
@@ -124,8 +127,13 @@ The following named parameters may be passed to C<new> or C<configure>:
 
 =item on_accept => CODE
 
-A callback that is invoked whenever a new client connects to the socket. If
-not supplied  the subclass method will be called instead.
+=item on_stream => CODE
+
+=item on_socket => CODE
+
+CODE reference for the event handlers. Because of the mutually-exclusive
+nature of their behaviour, only one of these may be set at a time. Setting one
+will remove the other two.
 
 =item handle => IO
 
@@ -135,33 +143,42 @@ The IO handle containing an existing listen-mode socket.
 
 =cut
 
+my @acceptor_events  = qw( on_accept on_stream on_socket );
+
 sub configure
 {
    my $self = shift;
    my %params = @_;
 
-   if( exists $params{on_accept} ) {
-      $self->{on_accept} = delete $params{on_accept};
+   if( grep exists $params{$_}, @acceptor_events ) {
+      grep( defined $_, @params{@acceptor_events} ) <= 1 or
+         croak "Can only set at most one of 'on_accept', 'on_stream' or 'on_socket'";
+
+      # Don't exists-test, so we'll clear the other two
+      $self->{$_} = delete $params{$_} for @acceptor_events;
    }
 
    croak "Cannot set 'on_read_ready' on a Listener" if exists $params{on_read_ready};
 
    if( exists $params{handle} ) {
       my $handle = delete $params{handle};
-      # Sanity check it
-      defined eval { $handle->sockname } or croak "IO handle $handle does not have a sockname";
+      # Sanity check it - it may be a bare GLOB ref, not an IO::Socket-derived handle
+      defined getsockname( $handle ) or croak "IO handle $handle does not have a sockname";
 
       # So now we know it's at least some kind of socket. Is it listening?
       # SO_ACCEPTCONN would tell us, but not all OSes implement it. Since it's
       # only a best-effort sanity check, we won't mind if the OS doesn't.
-      my $acceptconn = eval { $handle->sockopt( SO_ACCEPTCONN ) };
-      !defined $acceptconn or $acceptconn or croak "Socket is not accepting connections";
+      my $acceptconn = getsockopt( $handle, SOL_SOCKET, SO_ACCEPTCONN );
+      !defined $acceptconn or unpack( "I", $acceptconn ) or croak "Socket is not accepting connections";
+
+      # This is a bit naughty but hopefully nobody will mind...
+      bless $handle, "IO::Socket" if ref( $handle ) eq "GLOB";
 
       $self->SUPER::configure( read_handle => $handle );
    }
 
-   if( !$self->{on_accept} and !$self->can( 'on_accept' ) ) {
-      croak 'Expected either a on_accept callback or an ->on_accept method';
+   unless( grep $self->can_event( $_ ), @acceptor_events ) {
+      croak "Expected to be able to 'on_accept', 'on_stream' or 'on_socket'";
    }
 
    if( keys %params ) {
@@ -173,24 +190,32 @@ sub on_read_ready
 {
    my $self = shift;
 
-   my $newclient = $self->read_handle->accept();
+   my $socket = $self->read_handle;
+   my $handle = $socket->accept;
 
-   if( defined $newclient ) {
-      # TODO: make class/callback
-      if( $self->{on_accept} ) {
-         $self->{on_accept}->( $self, $newclient );
+   if( defined $handle ) {
+      $handle->blocking( 0 );
+
+      if( $self->can_event( "on_stream" ) ) {
+         # TODO: It doesn't make sense to put a SOCK_DGRAM in an
+         # IO::Async::Stream but currently we don't detect this
+         require IO::Async::Stream;
+         $self->invoke_event( on_stream => IO::Async::Stream->new( handle => $handle ) );
+      }
+      elsif( $self->can_event( "on_socket" ) ) {
+         require IO::Async::Socket;
+         $self->invoke_event( on_socket => IO::Async::Socket->new( handle => $handle ) );
+      }
+      # on_accept needs to be last in case of multiple layers of subclassing
+      elsif( $self->can_event( "on_accept" ) ) {
+         $self->invoke_event( on_accept => $handle );
       }
       else {
-         $self->on_accept( $newclient );
+         die "ARG! Missing on_accept,on_stream,on_socket!";
       }
    }
-   elsif( $! == EAGAIN ) {
-      # No client ready after all. Perhaps we're sharing the listen
-      # socket with other processes? Anyway; not fatal, just ignore it
-   }
-   else {
-      # TODO: make a callback
-      die "Cannot accept - $!";
+   elsif( $! != EAGAIN ) {
+      $self->maybe_invoke_event( on_accept_error => $socket, $! );
    }
 }
 
@@ -205,6 +230,12 @@ sub is_listening
    return ( defined $self->sockname );
 }
 
+=head2 $name = $listener->sockname
+
+Returns the C<sockname> of the underlying listening socket
+
+=cut
+
 sub sockname
 {
    my $self = shift;
@@ -213,6 +244,34 @@ sub sockname
    return $handle->sockname;
 }
 
+=head2 $family = $listener->family
+
+Returns the socket address family of the underlying listening socket
+
+=cut
+
+sub family
+{
+   my $self = shift;
+
+   my $sockname = $self->sockname or return undef;
+   return sockaddr_family( $sockname );
+}
+
+=head2 $socktype = $listener->socktype
+
+Returns the socket type of the underlying listening socket
+
+=cut
+
+sub socktype
+{
+   my $self = shift;
+
+   my $handle = $self->read_handle or return undef;
+   return $handle->sockopt(SO_TYPE);
+}
+
 =head2 $listener->listen( %params )
 
 This method sets up a listening socket using the addresses given, and will
@@ -236,17 +295,9 @@ is returned by the C<getaddrinfo> named resolver.
 =item addr => ARRAY
 
 Shortcut for passing a single address to listen on; it may be passed directly
-with this key, instead of in another array of its own.
-
-The address (or each element of the C<addrs> array) should be a reference to
-an array, with at least the following elements:
-
- [ $family, $socktype, $protocol, $address ]
-
-The first three arguments will be passed to a C<socket()> call and, if
-successful, the fourth to a C<bind()> call on the resulting socket. The socket
-will then be C<listen()>ed to put it into listening mode. Any trailing
-elements in this array will be ignored.
+with this key, instead of in another array of its own. This should be in a
+format recognised by L<IO::Async::Loop>'s C<extract_addrinfo> method. See also
+the C<EXAMPLES> section.
 
 =back
 
@@ -289,6 +340,13 @@ method.
 
 =back
 
+It is necessary to pass the C<socktype> hint to the resolver when resolving
+the host/service names into an address, as some OS's C<getaddrinfo> functions
+require this hint. A warning is emitted if neither C<socktype> nor C<protocol>
+hint is defined when performing a C<getaddrinfo> lookup. To avoid this warning
+while still specifying no particular C<socktype> hint (perhaps to invoke some
+OS-specific behaviour), pass C<0> as the C<socktype> value.
+
 In either case, the following keys are also taken:
 
 =over 8
@@ -302,9 +360,10 @@ Optional. A callback that is invoked when the listening socket is ready.
 =item on_listen_error => CODE
 
 A continuation this is invoked after all of the addresses have been tried, and
-none of them succeeded. Becasue there is no one error message that stands out
-as particularly noteworthy, none is given to this continuation. To track
-individual errors, see the C<on_fail> callback.
+none of them succeeded. It will be passed the most significant error that
+occurred, and the name of the operation it occurred in. Errors from the
+C<listen()> syscall are considered most significant, then C<bind()>, then
+C<sockopt()>, then finally C<socket()>.
 
 =item on_fail => CODE
 
@@ -332,6 +391,9 @@ be set. To prevent this, pass a false value such as 0.
 
 =back
 
+As a convenience, it also supports a C<handle> argument, which is passed
+directly to C<configure>.
+
 =cut
 
 sub listen
@@ -342,19 +404,25 @@ sub listen
    my $loop = $self->get_loop;
    defined $loop or croak "Cannot listen when not a member of a Loop"; # TODO: defer?
 
+   if( exists $params{handle} ) {
+      my $handle = $params{handle};
+      $self->configure( handle => $handle );
+      return;
+   }
+
    # Shortcut
    if( $params{addr} and not $params{addrs} ) {
       $params{addrs} = [ delete $params{addr} ];
    }
 
    my $on_listen = $params{on_listen}; # optional
-   !defined $on_listen or ref $on_listen eq "CODE" or croak "Expected 'on_listen' to be a CODE reference";
+   !defined $on_listen or ref $on_listen or croak "Expected 'on_listen' to be a reference";
 
    my $on_listen_error = $params{on_listen_error};
-   ref $on_listen_error eq "CODE" or croak "Expected 'on_listen_error' as a CODE reference";
+   ref $on_listen_error or croak "Expected 'on_listen_error' as a reference";
 
    my $on_fail = $params{on_fail};
-   !defined $on_fail or ref $on_fail eq "CODE" or croak "Expected 'on_fail' to be a CODE reference";
+   !defined $on_fail or ref $on_fail or croak "Expected 'on_fail' to be a reference";
 
    my $queuesize = $params{queuesize} || 3;
 
@@ -362,29 +430,35 @@ sub listen
       my $reuseaddr = 1;
       $reuseaddr = 0 if defined $params{reuseaddr} and not $params{reuseaddr};
 
+      my ( $listenerr, $binderr, $sockopterr, $socketerr );
+
       foreach my $addr ( @$addrlist ) {
-         my ( $family, $socktype, $proto, $address ) = @$addr;
+         my ( $family, $socktype, $proto, $address ) = $loop->extract_addrinfo( $addr );
 
          my $sock;
 
          unless( $sock = $loop->socket( $family, $socktype, $proto ) ) {
+            $socketerr = $!;
             $on_fail->( "socket", $family, $socktype, $proto, $! ) if $on_fail;
             next;
          }
 
          if( $reuseaddr ) {
             unless( $sock->sockopt( SO_REUSEADDR, 1 ) ) {
+               $sockopterr = $!;
                $on_fail->( "sockopt", $sock, SO_REUSEADDR, 1, $! ) if $on_fail;
                next;
             }
          }
 
          unless( $sock->bind( $address ) ) {
+            $binderr = $!;
             $on_fail->( "bind", $sock, $address, $! ) if $on_fail;
             next;
          }
 
          unless( $sock->listen( $queuesize ) ) {
+            $listenerr = $!;
             $on_fail->( "listen", $sock, $queuesize, $! ) if $on_fail;
             next;
          }
@@ -397,27 +471,33 @@ sub listen
       }
 
       # If we got this far, then none of the addresses succeeded
-      $on_listen_error->();
+      return $on_listen_error->( listen  => $listenerr  ) if $listenerr;
+      return $on_listen_error->( bind    => $binderr    ) if $binderr;
+      return $on_listen_error->( sockopt => $sockopterr ) if $sockopterr;
+      return $on_listen_error->( socket  => $socketerr  ) if $socketerr;
+      die 'Oops; $loop->listen failed but no error cause was found';
    }
 
    elsif( defined $params{service} ) {
       my $on_resolve_error = delete $params{on_resolve_error};
-      ref $on_resolve_error eq "CODE" or croak "Expected 'on_resolve_error' as CODE reference";
+      ref $on_resolve_error or croak "Expected 'on_resolve_error' as a reference";
 
       my $host = delete $params{host} || "";
 
       my $service = delete $params{service};
       defined $service or $service = ""; # might be 0
 
-      my $family   = delete $params{family} || 0;
-      my $socktype = delete $params{socktype} || 0;
-      my $protocol = delete $params{protocol} || 0;
+      my %gai_hints;
+      exists $params{$_} and $gai_hints{$_} = $params{$_} for qw( family socktype protocol flags );
 
-      my $flags = ( delete $params{flags} || 0 ) | AI_PASSIVE;
+      defined $gai_hints{socktype} or defined $gai_hints{protocol} or
+         carp "Attempting to ->listen without either 'socktype' or 'protocol' hint is not portable";
 
-      $loop->resolve(
-         type => 'getaddrinfo',
-         data => [ $host, $service, $family, $socktype, $protocol, $flags ],
+      $loop->resolver->getaddrinfo(
+         host    => $host,
+         service => $service,
+         passive => 1,
+         %gai_hints,
 
          on_resolved => sub {
             $self->listen( 
@@ -435,11 +515,85 @@ sub listen
    }
 }
 
-# Keep perl happy; keep Britain tidy
-1;
+=head1 EXAMPLES
 
-__END__
+=head2 Listening on UNIX Sockets
+
+The C<handle> argument can be passed an existing socket already in listening
+mode, making it possible to listen on other types of socket such as UNIX
+sockets.
+
+ use IO::Async::Listener;
+ use IO::Socket::UNIX;
+
+ use IO::Async::Loop;
+ my $loop = IO::Async::Loop->new();
+
+ my $listener = IO::Async::Listener->new(
+    on_stream => sub {
+       my ( undef, $stream ) = @_;
+
+       $stream->configure(
+          on_read => sub {
+             my ( $self, $buffref, $eof ) = @_;
+             $self->write( $$buffref );
+             $$buffref = "";
+             return 0;
+          },
+       );
+       
+       $loop->add( $stream );
+    },
+ );
+
+ $loop->add( $listener );
+
+ my $socket = IO::Socket::UNIX->new(
+    Local => "echo.sock",
+    Listen => 1,
+ ) or die "Cannot make UNIX socket - $!\n";
+
+ $listener->listen(
+    handle => $socket,
+ );
+
+ $loop->loop_forever;
+
+=head2 Passing Plain Socket Addresses
+
+The C<addr> or C<addrs> parameters should contain a definition of a plain
+socket address in a form that the L<IO::Async::Loop> C<extract_addrinfo>
+method can use.
+
+This example shows how to use the C<Socket> functions to construct one for
+TCP port 8001 on address 10.0.0.1:
+
+ $listener->listen(
+    addr => {
+       family   => "inet",
+       socktype => "stream",
+       port     => 8001,
+       ip       => "10.0.0.1",
+    },
+    ...
+ );
+
+This example shows another way to listen on a UNIX socket, similar to the
+earlier example:
+
+ $listener->listen(
+    addr => {
+       family   => "unix",
+       socktype => "stream",
+       path     => "echo.sock",
+    },
+    ...
+ );
 
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
