@@ -8,7 +8,7 @@ package IO::Async::MergePoint;
 use strict;
 use warnings;
 
-our $VERSION = '0.30';
+our $VERSION = '0.42';
 
 use Carp;
 
@@ -22,7 +22,7 @@ C<IO::Async::MergePoint> - resynchronise diverged control flow
 
 This module as now been moved to its own dist of L<Async::MergePoint>.
 
-It is kept here as a trivial subclass for backward compatibilty. Eventually
+It is kept here as a trivial subclass for backward compatibility. Eventually
 this subclass may be removed. Any code using C<IO::Async::MergePoint> should
 instead use L<Async::MergePoint>.
 
@@ -70,11 +70,10 @@ contain the results of that step. The C<on_finished> callback is passed a hash
 
 =cut
 
-# Keep perl happy; keep Britain tidy
-1;
-
-__END__
-
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
