@@ -1,18 +1,21 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2006-2010 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2006-2011 -- leonerd@leonerd.org.uk
 
 package IO::Async::Notifier;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.30';
+our $VERSION = '0.42';
 
 use Carp;
 use Scalar::Util qw( weaken );
 
+# Perl 5.8.4 cannot do trampolines by modiying @_ then goto &$code
+use constant HAS_BROKEN_TRAMPOLINES => ( $] == "5.008004" );
+
 =head1 NAME
 
 C<IO::Async::Notifier> - base class for C<IO::Async> event objects
@@ -32,14 +35,16 @@ Usually not directly used by a program, but one valid use case may be:
  my $notifier = IO::Async::Notifier->new();
 
  $notifier->add_child(
-    IO::Async::Stream->new(
-       read_handle => \*STDIN,
+    IO::Async::Stream->new_for_stdin(
        on_read => sub {
           my $self = shift;
-          my ( $buffref, $closed ) = @_;
-          $$buffref =~ s/^(.*)\n// or return 0;
-          print "You said $1\n";
-          return 1;
+          my ( $buffref, $eof ) = @_;
+
+          while( $$buffref =~ s/^(.*)\n// ) {
+             print "You said $1\n";
+          }
+
+          return 0;
        },
     )
  );
@@ -97,6 +102,14 @@ L<IO::Async::Timer> - base class for Notifiers that use timed delays
 
 L<IO::Async::Signal> - event callback on receipt of a POSIX signal
 
+=item *
+
+L<IO::Async::PID> - event callback on exit of a child process
+
+=item *
+
+L<IO::Async::Process> - start and manage a child process
+
 =back
 
 For more detail, see the SYNOPSIS section in one of the above.
@@ -119,6 +132,17 @@ details on the parameters that exist, and their uses. Some parameters may only
 support being set once at construction time, or only support being changed if
 the object is in a particular state.
 
+The following parameters are supported by all Notifiers:
+
+=over 8
+
+=item notifier_name => STRING
+
+Optional string used to identify this particular Notifier. This value will be
+returned by the C<notifier_name> method.
+
+=back
+
 =cut
 
 =head1 CONSTRUCTOR
@@ -131,16 +155,9 @@ This function returns a new instance of a C<IO::Async::Notifier> object with
 the given initial values of the named parameters.
 
 Up until C<IO::Async> version 0.19, this module used to implement the IO
-handle features now found in the C<IO::Async::Handle> subclass. To allow for a
-smooth upgrade of existing code, this constructor check for any C<%params> key
-which looks like it belongs there instead. These keys are C<handle>,
-C<read_handle>, C<write_handle>, C<on_read_ready> and C<on_write_ready>. If
-any of these keys are present, then a C<IO::Async::Handle> is returned.
-
-Do not rely on this feature in new code.  This logic exists purely to provide
-an upgrade path from older code that still expects C<IO::Async::Notifier> to
-provide filehandle operations. This produces a deprecation warning. At some
-point in the future this functionallity may be removed.
+handle features now found in the C<IO::Async::Handle> subclass. Code that
+needs to use any of C<handle>, C<read_handle>, C<write_handle>,
+C<on_read_ready> or C<on_write_ready> should use L<IO::Async::Handle> instead.
 
 =cut
 
@@ -169,6 +186,10 @@ sub new
    return $self;
 }
 
+=head1 METHODS
+
+=cut
+
 =head2 $notifier->configure( %params )
 
 Adjust the named parameters of the C<Notifier> as given by the C<%params>
@@ -182,6 +203,10 @@ sub configure
    my $self = shift;
    my %params = @_;
 
+   foreach (qw( notifier_name )) {
+      $self->{$_} = delete $params{$_} if exists $params{$_};
+   }
+
    # We don't recognise any configure keys at this level
    if( keys %params ) {
       my $class = ref $self;
@@ -219,6 +244,20 @@ sub __set_loop
    $self->_add_to_loop( $self->{loop} ) if $self->{loop};
 }
 
+=head2 $name = $notifier->notifier_name
+
+Returns the name to identify this Notifier. If a has not been set, it will
+return the empty string. Subclasses may wish to override this behaviour to
+return some more useful information, perhaps from configured parameters.
+
+=cut
+
+sub notifier_name
+{
+   my $self = shift;
+   return $self->{notifier_name} || "";
+}
+
 =head1 CHILD NOTIFIERS
 
 During the execution of a program, it may be the case that certain IO handles
@@ -314,6 +353,18 @@ sub remove_child
    }
 }
 
+sub _remove_from_outer
+{
+   my $self = shift;
+
+   if( my $parent = $self->parent ) {
+      $parent->remove_child( $self );
+   }
+   elsif( my $loop = $self->get_loop ) {
+      $loop->remove( $self );
+   }
+}
+
 =head1 SUBCLASS METHODS
 
 C<IO::Async::Notifier> is a base class provided so that specific subclasses of
@@ -410,6 +461,22 @@ the CODE ref internally for efficiency.
 
 The C<$code> argument may also be a plain string, which will be used as a
 method name; the returned CODE ref will then invoke that method on the object.
+In this case the method name is stored symbolically in the returned CODE
+reference, and dynamically dispatched each time the reference is invoked. This
+allows it to follow code reloading, dynamic replacement of class methods, or
+other similar techniques.
+
+If the C<$mref> CODE reference is being stored in some object other than the
+one it refers to, remember that since the Notifier is only weakly captured, it
+is possible that it has been destroyed by the time the code runs, and so the
+reference will be passed as C<undef>. This should be protected against by the
+code body.
+
+ $other_object->{on_event} = $notifier->_capture_weakself( sub {
+    my $notifier = shift or return;
+    my ( @event_args ) = @_;
+    ...
+ } );
 
 =cut
 
@@ -420,25 +487,190 @@ sub _capture_weakself
 
    if( !ref $code ) {
       my $class = ref $self;
-      my $coderef = $self->can( $code ) or
+      # Don't save this coderef, or it will break dynamic method dispatch,
+      # which means code reloading, dynamic replacement, or other funky
+      # techniques stop working
+      $self->can( $code ) or
          croak qq(Can't locate object method "$code" via package "$class");
-
-      $code = $coderef;
    }
 
    weaken $self;
 
    return sub {
-      unshift @_, $self;
-      goto &$code;
+      my $cv = ref( $code ) ? $code : $self->can( $code );
+
+      if( HAS_BROKEN_TRAMPOLINES ) {
+         return $cv->( $self, @_ );
+      }
+      else {
+         unshift @_, $self;
+         goto &$cv;
+      }
    };
 }
 
-# Keep perl happy; keep Britain tidy
-1;
+=head2 $mref = $notifier->_replace_weakself( $code )
 
-__END__
+Returns a new CODE ref which, when invoked, will invoke the originally-passed
+ref, with a reference to the Notifier replacing its first argument. The
+Notifier reference is stored weakly in C<$mref>, so this CODE ref may be
+stored in the Notifier itself without creating a cycle.
+
+For example,
+
+ my $mref = $notifier->_replace_weakself( sub {
+    my ( $notifier, $arg ) = @_;
+    print "Notifier $notifier got argument $arg\n";
+ } );
+
+ $mref->( $object, 123 );
+
+This is provided as a utility for Notifier subclasses to use for event
+callbacks on other objects, where the delegated object is passed in the
+function's arguments.
+
+The C<$code> argument may also be a plain string, which will be used as a
+method name; the returned CODE ref will then invoke that method on the object.
+As with C<_capture_weakself> this is stored symbolically.
+
+As with C<_capture_weakself>, care should be taken against Notifier
+destruction if the C<$mref> CODE reference is stored in some other object.
+
+=cut
+
+sub _replace_weakself
+{
+   my $self = shift;
+   my ( $code ) = @_;   # actually bare method names work too
+
+   if( !ref $code ) {
+      # Don't save this coderef, see _capture_weakself for why
+      my $class = ref $self;
+      $self->can( $code ) or
+         croak qq(Can't locate object method "$code" via package "$class");
+   }
+
+   weaken $self;
+
+   return sub {
+      my $cv = ref( $code ) ? $code : $self->can( $code );
+
+      if( HAS_BROKEN_TRAMPOLINES ) {
+         return $cv->( $self, @_[1..$#_] );
+      }
+      else {
+         # Don't assign to $_[0] directly or we will change caller's first argument
+         shift @_;
+         unshift @_, $self;
+         goto &$cv;
+      }
+   };
+}
+
+=head2 $code = $notifier->can_event( $event_name )
+
+Returns a C<CODE> reference if the object can perform the given event name,
+either by a configured C<CODE> reference parameter, or by implementing a
+method. If the object is unable to handle this event, C<undef> is returned.
+
+=cut
+
+sub can_event
+{
+   my $self = shift;
+   my ( $event_name ) = @_;
+
+   return $self->{$event_name} || $self->can( $event_name );
+}
+
+=head2 $callback = $notifier->make_event_cb( $event_name )
+
+Returns a C<CODE> reference which, when invoked, will execute the given event
+handler. Event handlers may either be subclass methods, or parameters given to
+the C<new> or C<configure> method.
+
+The event handler can be passed extra arguments by giving them to the C<CODE>
+reference; the first parameter received will be a reference to the notifier
+itself. This is stored weakly in the closure, so it is safe to store the
+resulting C<CODE> reference in the object itself without causing a reference
+cycle.
+
+=cut
+
+sub make_event_cb
+{
+   my $self = shift;
+   my ( $event_name ) = @_;
+
+   my $code = $self->can_event( $event_name )
+      or croak "$self cannot handle $event_name event";
+
+   return $self->_capture_weakself( $code );
+}
+
+=head2 $callback = $notifier->maybe_make_event_cb( $event_name )
+
+Similar to C<make_event_cb> but will return C<undef> if the object cannot
+handle the named event, rather than throwing an exception.
+
+=cut
+
+sub maybe_make_event_cb
+{
+   my $self = shift;
+   my ( $event_name ) = @_;
+
+   my $code = $self->can_event( $event_name )
+      or return undef;
+
+   return $self->_capture_weakself( $code );
+}
+
+=head2 @ret = $notifier->invoke_event( $event_name, @args )
+
+Invokes the given event handler, passing in the given arguments. Event
+handlers may either be subclass methods, or parameters given to the C<new> or
+C<configure> method. Returns whatever the underlying method or CODE reference
+returned.
+
+=cut
+
+sub invoke_event
+{
+   my $self = shift;
+   my ( $event_name, @args ) = @_;
+
+   my $code = $self->can_event( $event_name )
+      or croak "$self cannot handle $event_name event";
+
+   return $code->( $self, @args );
+}
+
+=head2 $retref = $notifier->maybe_invoke_event( $event_name, @args )
+
+Similar to C<invoke_event> but will return C<undef> if the object cannot
+handle the name event, rather than throwing an exception. In order to
+distinguish this from an event-handling function that simply returned
+C<undef>, if the object does handle the event, the list that it returns will
+be returned in an ARRAY reference.
+
+=cut
+
+sub maybe_invoke_event
+{
+   my $self = shift;
+   my ( $event_name, @args ) = @_;
+
+   my $code = $self->can_event( $event_name )
+      or return undef;
+
+   return [ $code->( $self, @args ) ];
+}
 
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
