@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2011 -- leonerd@leonerd.org.uk
 
 package IO::Async::Signal;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.29';
+our $VERSION = '0.42';
 
 use Carp;
 
@@ -38,32 +38,22 @@ C<IO::Async::Signal> - event callback on receipt of a POSIX signal
 
 =head1 DESCRIPTION
 
-This module provides a class of C<IO::Async::Notifier> which invokes its
-callback when a particular POSIX signal is received.
+This subclass of L<IO::Async::Notifier> invokes its callback when a particular
+POSIX signal is received.
 
 Multiple objects can be added to a C<Loop> that all watch for the same signal.
 The callback functions will all be invoked, in no particular order.
 
-This object may be used in one of two ways; with a callback function, or as a
-base class.
+=cut
 
-=over 4
+=head1 EVENTS
 
-=item Callbacks
+The following events are invoked, either using subclass methods or CODE
+references in parameters:
 
-If the C<on_receipt> key is supplied to the constructor, it should contain a
-CODE reference to a callback function to be invoked when the signal is received.
-
- $on_receipt->( $self )
-
-=item Base Class
-
-If a subclass is built, then it can override the C<on_receipt> method.
-
- $self->on_receipt()
-
-=back
+=head2 on_receipt
 
+Invoked when the signal is received.
 
 =cut
 
@@ -80,8 +70,7 @@ only be given at construction time.
 
 =item on_receipt => CODE
 
-CODE reference to callback to invoke when the signal is received. If not
-supplied, the subclass method will be called instead.
+CODE reference for the C<on_receipt> event.
 
 =back
 
@@ -120,7 +109,7 @@ sub configure
       }
    }
 
-   if( !$self->{on_receipt} and !$self->can( 'on_receipt' ) ) {
+   unless( $self->can_event( 'on_receipt' ) ) {
       croak 'Expected either a on_receipt callback or an ->on_receipt method';
    }
 
@@ -132,14 +121,7 @@ sub _add_to_loop
    my $self = shift;
    my ( $loop ) = @_;
 
-   if( !$self->{cb} ) {
-      if( $self->{on_receipt} ) {
-         $self->{cb} = $self->_capture_weakself( $self->{on_receipt} );
-      }
-      else {
-         $self->{cb} = $self->_capture_weakself( 'on_receipt' );
-      }
-   }
+   $self->{cb} ||= $self->make_event_cb( 'on_receipt' );
 
    $self->{id} = $loop->attach_signal( $self->{name}, $self->{cb} );
 }
@@ -153,11 +135,20 @@ sub _remove_from_loop
    undef $self->{id};
 }
 
-# Keep perl happy; keep Britain tidy
-1;
+sub notifier_name
+{
+   my $self = shift;
+   if( length( my $name = $self->SUPER::notifier_name ) ) {
+      return $name;
+   }
 
-__END__
+   return $self->{name};
+}
 
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
