@@ -1,26 +1,33 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2006-2010 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2006-2011 -- leonerd@leonerd.org.uk
 
 package IO::Async::Stream;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.30';
+our $VERSION = '0.42';
 
 use base qw( IO::Async::Handle );
 
-use POSIX qw( EAGAIN EWOULDBLOCK EINTR );
+use POSIX qw( EAGAIN EWOULDBLOCK EINTR EPIPE );
 
 use Carp;
 
+use Encode 2.11 qw( find_encoding STOP_AT_PARTIAL );
+
 # Tuneable from outside
 # Not yet documented
 our $READLEN  = 8192;
 our $WRITELEN = 8192;
 
+# Indicies in writequeue elements
+use constant WQ_DATA     => 0;
+use constant WQ_ON_FLUSH => 1;
+use constant WQ_GENSUB   => 2;
+
 =head1 NAME
 
 C<IO::Async::Stream> - event callbacks and write bufering for a stream
@@ -28,32 +35,24 @@ filehandle
 
 =head1 SYNOPSIS
 
- use IO::Socket::INET;
  use IO::Async::Stream;
 
  use IO::Async::Loop;
  my $loop = IO::Async::Loop->new();
 
- my $socket = IO::Socket::INET->new(
-    PeerHost => "some.other.host",
-    PeerPort => 12345,
-    Blocking => 0,                   # This line is very important
- );
-
  my $stream = IO::Async::Stream->new(
-    handle => $socket,
+    read_handle  => \*STDIN,
+    write_handle => \*STDOUT,
 
     on_read => sub {
-       my ( $self, $buffref, $closed ) = @_;
+       my ( $self, $buffref, $eof ) = @_;
 
-       if( $$buffref =~ s/^(.*\n)// ) {
+       while( $$buffref =~ s/^(.*\n)// ) {
           print "Received a line $1";
-
-          return 1;
        }
 
-       if( $closed ) {
-          print "Closed; last partial line is $$buffref\n";
+       if( $eof ) {
+          print "EOF; last partial line is $$buffref\n";
        }
 
        return 0;
@@ -68,7 +67,7 @@ Or
     handle => ...,
 
     on_read => sub {
-       my ( $self, $buffref, $closed ) = @_;
+       my ( $self, $buffref, $eof ) = @_;
 
        if( length $$buffref >= 16 ) {
           my $record = substr( $$buffref, 0, 16, "" );
@@ -77,24 +76,14 @@ Or
           return 1;
        }
 
-       if( $closed and length $$buffref ) {
-          print "Closed: a partial record still exists\n";
+       if( $eof and length $$buffref ) {
+          print "EOF: a partial record still exists\n";
        }
 
        return 0;
     }
  );
 
-Or
-
- use IO::Handle;
-
- my $stream = IO::Async::Stream->new(
-    read_handle  => \*STDIN,
-    write_handle => \*STDOUT,
-    ...
- );
-
 =head1 DESCRIPTION
 
 This subclass of L<IO::Async::Handle> contains a filehandle that represents
@@ -102,46 +91,39 @@ a byte-stream. It provides buffering for both incoming and outgoing data. It
 invokes the C<on_read> handler when new data is read from the filehandle. Data
 may be written to the filehandle by calling the C<write()> method.
 
-This object may be used in one of two ways; as an instance with CODE
-references as callbacks, or as a base class with overridden methods.
+For implementing real network protocols that are based on messages sent over a
+byte-stream (such as a TCP socket), it may be more appropriate to use a
+subclass of L<IO::Async::Protocol::Stream>.
 
-=over 4
+=cut
 
-=item Subclassing
+=head1 EVENTS
 
-If a subclass is built, then it can override the following methods to handle
-events:
+The following events are invoked, either using subclass methods or CODE
+references in parameters:
 
- $ret = $self->on_read( \$buffer, $handleclosed )
+=head2 $ret = on_read \$buffer, $eof
 
- $self->on_read_error( $errno )
+Invoked when more data is available in the internal receiving buffer.
 
- $self->on_outgoing_empty()
-
- $self->on_write_error( $errno )
-
-=back
-
-The first argument to C<on_read()> is a reference to a plain perl string. The
-code should inspect and remove any data it likes, but is not required to
-remove all, or indeed any of the data. Any data remaining in the buffer will
-be preserved for the next call, the next time more data is received from the
-handle.
+The first argument is a reference to a plain perl string. The code should
+inspect and remove any data it likes, but is not required to remove all, or
+indeed any of the data. Any data remaining in the buffer will be preserved for
+the next call, the next time more data is received from the handle.
 
 In this way, it is easy to implement code that reads records of some form when
 completed, but ignores partially-received records, until all the data is
-present. If the method is confident no more useful data remains, it should
-return C<0>. If not, it should return C<1>, and the method will be called
+present. If the handler is confident no more useful data remains, it should
+return C<0>. If not, it should return C<1>, and the handler will be called
 again. This makes it easy to implement code that handles multiple incoming
 records at the same time. See the examples at the end of this documentation
 for more detail.
 
-The second argument to the C<on_read()> method is a scalar indicating whether
-the handle has been closed. Normally it is false, but will become true once
-the handle closes. A reference to the buffer is passed to the method in the
-usual way, so it may inspect data contained in it. Once the method returns a
-false value, it will not be called again, as the handle is now closed and no
-more data can arrive.
+The second argument is a scalar indicating whether the stream has reported an
+end-of-file (EOF) condition. A reference to the buffer is passed to the
+handler in the usual way, so it may inspect data contained in it. Once the
+handler returns a false value, it will not be called again, as the handle is
+now at EOF and no more data can arrive.
 
 The C<on_read()> code may also dynamically replace itself with a new callback
 by returning a CODE reference instead of C<0> or C<1>. The original callback
@@ -150,16 +132,38 @@ C<undef>. Whenever the callback is changed in this way, the new code is called
 again; even if the read buffer is currently empty. See the examples at the end
 of this documentation for more detail.
 
+=head2 on_read_eof
+
+Optional. Invoked when the read handle indicates an end-of-file (EOF)
+condition. If there is any data in the buffer still to be processed, the
+C<on_read> event will be invoked first, before this one.
+
+=head2 on_write_eof
+
+Optional. Invoked when the write handle indicates an end-of-file (EOF)
+condition. Note that this condition can only be detected after a C<write>
+syscall returns the C<EPIPE> error. If there is no data pending to be written
+then it will not be detected yet.
+
+=head2 on_read_error $errno
+
+Optional. Invoked when the C<sysread()> method on the read handle fails.
+
+=head2 on_write_error $errno
+
+Optional. Invoked when the C<syswrite()> method on the write handle fails.
+
 The C<on_read_error> and C<on_write_error> handlers are passed the value of
 C<$!> at the time the error occured. (The C<$!> variable itself, by its
 nature, may have changed from the original error by the time this handler
 runs so it should always use the value passed in).
 
 If an error occurs when the corresponding error callback is not supplied, and
-there is not a subclass method for it, then the C<close()> method is
-called instead.
+there is not a handler for it, then the C<close()> method is called instead.
 
-The C<on_outgoing_empty> handler is not passed any arguments.
+=head2 on_outgoing_empty
+
+Optional. Invoked when the writing data buffer becomes empty.
 
 =cut
 
@@ -167,11 +171,13 @@ sub _init
 {
    my $self = shift;
 
-   $self->{writebuff} = "";
+   $self->{writequeue} = []; # Queue of ARRAYs. Each will be [ $data, $on_flushed, $gensub ]
    $self->{readbuff} = "";
 
    $self->{read_len}  = $READLEN;
    $self->{write_len} = $WRITELEN;
+
+   $self->{close_on_read_eof} = 1;
 }
 
 =head1 PARAMETERS
@@ -194,30 +200,13 @@ Shortcut to specifying the same IO handle for both of the above.
 
 =item on_read => CODE
 
-A CODE reference for when more data is available in the internal receiving 
-buffer.
-
- $ret = $on_read->( $self, \$buffer, $handleclosed )
-
 =item on_read_error => CODE
 
-Optional. A CODE reference for when the C<sysread()> method on the read handle
-fails.
-
- $on_read_error->( $self, $errno )
-
 =item on_outgoing_empty => CODE
 
-Optional. A CODE reference for when the writing data buffer becomes empty.
-
- $on_outgoing_empty->( $self )
-
 =item on_write_error => CODE
 
-Optional. A CODE reference for when the C<syswrite()> method on the write
-handle fails.
-
- $on_write_error->( $self, $errno )
+CODE references for event handlers.
 
 =item autoflush => BOOL
 
@@ -257,6 +246,34 @@ Optional. Analogous to the C<read_all> option, but for writing. When
 C<autoflush> is enabled, this option only affects deferred writing if the
 initial attempt failed due to buffer space.
 
+=item close_on_read_eof => BOOL
+
+Optional. Usually true, but if set to a false value then the stream will not
+be C<close>d when an EOF condition occurs on read. This is normally not useful
+as at that point the underlying stream filehandle is no longer useable, but it
+may be useful for reading regular files, or interacting with TTY devices.
+
+=item encoding => STRING
+
+If supplied, sets the name of encoding of the underlying stream. If an
+encoding is set, then the C<print> method will expect to receive Unicode
+strings and encodes them into bytes, and incoming bytes will be decoded into
+Unicode strings for the C<on_read> event.
+
+If an encoding is not supplied then C<print> and C<on_read> will work in byte
+strings.
+
+I<IMPORTANT NOTE:> in order to handle reads of UTF-8 content or other
+multibyte encodings, the code implementing the C<on_read> event uses a feature
+of L<Encode>; the C<STOP_AT_PARTIAL> flag. While this flag has existed for a
+while and is used by the C<:encoding> PerlIO layer itself for similar
+purposes, the flag is not officially documented by the C<Encode> module. In
+principle this undocumented feature could be subject to change, in practice I
+believe it to be reasonably stable.
+
+This note applies only to the C<on_read> event; data written using the
+C<write> method does not rely on any undocumented features of C<Encode>.
+
 =back
 
 If a read handle is given, it is required that either an C<on_read> callback
@@ -281,15 +298,23 @@ sub configure
    my $self = shift;
    my %params = @_;
 
-   for (qw( on_read on_outgoing_empty on_read_error on_write_error
-            autoflush read_len read_all write_len write_all )) {
+   for (qw( on_read on_outgoing_empty on_read_eof on_write_eof on_read_error
+            on_write_error autoflush read_len read_all write_len write_all
+            close_on_read_eof )) {
       $self->{$_} = delete $params{$_} if exists $params{$_};
    }
 
+   if( exists $params{encoding} ) {
+      my $encoding = delete $params{encoding};
+      my $obj = find_encoding( $encoding );
+      defined $obj or croak "Cannot handle an encoding of '$encoding'";
+      $self->{encoding} = $obj;
+   }
+
    $self->SUPER::configure( %params );
 
    if( $self->get_loop and $self->read_handle ) {
-      $self->{on_read} or $self->can( "on_read" ) or
+      $self->can_event( "on_read" ) or
          croak 'Expected either an on_read callback or to be able to ->on_read';
    }
 }
@@ -299,11 +324,15 @@ sub _add_to_loop
    my $self = shift;
 
    if( defined $self->read_handle ) {
-      $self->{on_read} or $self->can( "on_read" ) or
+      $self->can_event( "on_read" ) or
          croak 'Expected either an on_read callback or to be able to ->on_read';
    }
 
    $self->SUPER::_add_to_loop( @_ );
+
+   if( !$self->_is_empty ) {
+      $self->want_writeready( 1 );
+   }
 }
 
 =head1 METHODS
@@ -320,6 +349,60 @@ sub _nonfatal_error
           $errno == EINTR;
 }
 
+sub _is_empty
+{
+   my $self = shift;
+   return !@{ $self->{writequeue} };
+}
+
+sub _flush_one
+{
+   my $self = shift;
+
+   my $head = $self->{writequeue}[WQ_DATA];
+
+   if( !defined $head->[WQ_DATA] ) {
+      my $gensub = $head->[WQ_GENSUB] or die "Internal consistency problem - empty writequeue item without a gensub\n";
+      $head->[WQ_DATA] = $gensub->( $self );
+
+      if( !defined $head->[WQ_DATA] ) {
+         $head->[WQ_ON_FLUSH]->( $self ) if $head->[WQ_ON_FLUSH];
+         shift @{ $self->{writequeue} };
+
+         return 1;
+      }
+   }
+
+   my $len = $self->write_handle->syswrite( $head->[WQ_DATA], $self->{write_len} );
+
+   if( !defined $len ) {
+      my $errno = $!;
+
+      return 0 if _nonfatal_error( $errno );
+
+      $self->maybe_invoke_event( on_write_eof => ) if $errno == EPIPE;
+
+      $self->maybe_invoke_event( on_write_error => $errno )
+         or $self->close_now;
+
+      return 0;
+   }
+
+   substr( $head->[WQ_DATA], 0, $len ) = "";
+
+   if( !length $head->[WQ_DATA] ) {
+      if( $head->[WQ_GENSUB] ) {
+         undef $head->[WQ_DATA]; # We'll get some more next time around
+      }
+      else {
+         $head->[WQ_ON_FLUSH]->( $self ) if $head->[WQ_ON_FLUSH];
+         shift @{ $self->{writequeue} };
+      }
+   }
+
+   return 1;
+}
+
 =head2 $stream->close
 
 A synonym for C<close_when_empty>. This should not be used when the deferred
@@ -353,7 +436,7 @@ sub close_when_empty
 {
    my $self = shift;
 
-   return $self->SUPER::close if length( $self->{writebuff} ) == 0;
+   return $self->SUPER::close if $self->_is_empty;
 
    $self->{stream_closing} = 1;
 }
@@ -370,13 +453,13 @@ sub close_now
 {
    my $self = shift;
 
-   $self->{writebuff} = "";
+   undef @{ $self->{writequeue} };
    undef $self->{stream_closing};
 
    $self->SUPER::close;
 }
 
-=head2 $stream->write( $data )
+=head2 $stream->write( $data, %params )
 
 This method adds data to the outgoing data queue, or writes it immediately,
 according to the C<autoflush> parameter.
@@ -387,35 +470,103 @@ will have been written by the time this method returns. If it fails to write
 completely, then the data is queued as if C<autoflush> were not set, and will
 be flushed as normal.
 
+C<$data> can either be a plain string, or a CODE reference. If it is a CODE
+reference, it will be invoked to generate data to be written. Each time the
+filehandle is ready to receive more data to it, the function is invoked, and
+what it returns written to the filehandle. Once the function has finished
+generating data it should return undef. The function is passed the Stream
+object as its first argument.
+
+For example, to stream the contents of an existing opened filehandle:
+
+ open my $fileh, "<", $path or die "Cannot open $path - $!";
+
+ $stream->write( sub {
+    my ( $stream ) = @_;
+
+    sysread $fileh, my $buffer, 8192 or return;
+    return $buffer;
+ } );
+
+Takes the following optional named parameters in C<%params>:
+
+=over 8
+
+=item on_flush => CODE
+
+A CODE reference which will be invoked once the data queued by this C<write>
+call has been flushed. This will be invoked even if the buffer itself is not
+yet empty; if more data has been queued since the call.
+
+ $on_flush->( $stream )
+
+=back
+
+If the object is not yet a member of a loop and doesn't yet have a
+C<write_handle>, then calls to the C<write> method will simply queue the data
+and return. It will be flushed when the object is added to the loop.
+
+If C<$data> is a defined but empty string, the write is still queued, and the
+C<on_flush> continuation will be invoked, if supplied. This can be used to
+obtain a marker, to invoke some code once the output queue has been flushed up
+to this point.
+
 =cut
 
 sub write
 {
    my $self = shift;
-   my ( $data ) = @_;
+   my ( $data, %params ) = @_;
 
    carp "Cannot write data to a Stream that is closing" and return if $self->{stream_closing};
-   croak "Cannot write data to a Stream with no write_handle" unless my $handle = $self->write_handle;
 
-   if( $self->{autoflush} ) {
-      $data = $self->{writebuff} . $data if length $self->{writebuff};
+   # Allow writes without a filehandle if we're not yet in a Loop, just don't
+   # try to flush them
+   my $handle = $self->write_handle;
 
-      while( length $data ) {
-         my $len = $handle->syswrite( $data, $self->{write_len} );
+   croak "Cannot write data to a Stream with no write_handle" if !$handle and $self->get_loop;
 
-         last if !$len; # stop on any errors and defer back to the non-autoflush path
+   if( my $encoding = $self->{encoding} ) {
+      $data = $encoding->encode( $data );
+   }
 
-         substr( $data, 0, $len ) = "";
+   # Combine short writes we can
+   my $tail = @{ $self->{writequeue} } ? $self->{writequeue}[-1] : undef;
+
+   if( $tail and
+       !ref $data and
+       !$tail->[WQ_GENSUB] and
+       length($data) + length($tail->[WQ_DATA]) < $self->{write_len} and
+       !$params{on_flush} and
+       !$tail->[WQ_ON_FLUSH]) {
+      $tail->[WQ_DATA] .= $data;
+   }
+   else {
+      push @{ $self->{writequeue} }, my $elem = [];
+
+      if( ref $data eq "CODE" ) {
+         $elem->[WQ_GENSUB] = $data;
+      }
+      else {
+         $elem->[WQ_DATA] = $data;
       }
 
-      if( !length $data ) {
+      $elem->[WQ_ON_FLUSH] = delete $params{on_flush};
+   }
+
+   keys %params and croak "Unrecognised keys for ->write - " . join( ", ", keys %params );
+
+   return unless $handle;
+
+   if( $self->{autoflush} ) {
+      1 while !$self->_is_empty and $self->_flush_one;
+
+      if( $self->_is_empty ) {
          $self->want_writeready( 0 );
          return;
       }
    }
 
-   $self->{writebuff} .= $data;
-
    $self->want_writeready( 1 );
 }
 
@@ -434,29 +585,30 @@ sub on_read_ready
 
          return if _nonfatal_error( $errno );
 
-         if( defined $self->{on_read_error} ) {
-            $self->{on_read_error}->( $self, $errno );
-         }
-         elsif( $self->can( "on_read_error" ) ) {
-            $self->on_read_error( $errno );
-         }
-         else {
-            $self->close_now;
-         }
+         $self->maybe_invoke_event( on_read_error => $errno )
+            or $self->close_now;
 
          return;
       }
 
-      my $handleclosed = ( $len == 0 );
+      my $eof = ( $len == 0 );
 
-      $self->{readbuff} .= $data if( !$handleclosed );
+      if( my $encoding = $self->{encoding} ) {
+         my $bytes = defined $self->{bytes_remaining} ? $self->{bytes_remaining} . $data : $data;
+         $data = $encoding->decode( $bytes, STOP_AT_PARTIAL );
+         $self->{bytes_remaining} = $bytes;
+      }
+
+      $self->{readbuff} .= $data if !$eof;
 
       while(1) {
-         my $on_read = $self->{current_on_read}
-                        || $self->{on_read}
-                        || $self->can( "on_read" );
-
-         my $ret = $on_read->( $self, \$self->{readbuff}, $handleclosed );
+         my $ret;
+         if( my $on_read = $self->{current_on_read} ) {
+            $ret = $on_read->( $self, \$self->{readbuff}, $eof );
+         }
+         else {
+            $ret = $self->invoke_event( on_read => \$self->{readbuff}, $eof );
+         }
 
          my $again;
 
@@ -469,13 +621,17 @@ sub on_read_ready
             $again = 1;
          }
          else {
-            $again = $ret && ( length( $self->{readbuff} ) > 0 || $handleclosed );
+            $again = $ret && ( length( $self->{readbuff} ) > 0 || $eof );
          }
 
          last if !$again;
       }
 
-      $self->close_now, return if $handleclosed;
+      if( $eof ) {
+         $self->maybe_invoke_event( on_read_eof => );
+         $self->close_now if $self->{close_on_read_eof};
+         return;
+      }
 
       last unless $self->{read_all};
    }
@@ -485,56 +641,37 @@ sub on_write_ready
 {
    my $self = shift;
 
-   my $handle = $self->write_handle;
-
-   while( length $self->{writebuff} ) {
-      my $len = $handle->syswrite( $self->{writebuff}, $self->{write_len} );
-
-      if( !defined $len ) {
-         my $errno = $!;
-
-         return if _nonfatal_error( $errno );
-
-         if( defined $self->{on_write_error} ) {
-            $self->{on_write_error}->( $self, $errno );
-         }
-         elsif( $self->can( "on_write_error" ) ) {
-            $self->on_write_error( $errno );
-         }
-         else {
-            $self->close_now;
-         }
-
-         return;
-      }
-
-      if( $len == 0 ) {
-         $self->close_now;
-         return;
-      }
-
-      substr( $self->{writebuff}, 0, $len ) = "";
-
-      last unless $self->{write_all};
-   }
+   1 while !$self->_is_empty and $self->_flush_one and $self->{write_all};
 
    # All data successfully flushed
-   if( length( $self->{writebuff} ) == 0 ) {
+   if( $self->_is_empty ) {
       $self->want_writeready( 0 );
 
-      my $on_outgoing_empty = $self->{on_outgoing_empty}
-                               || $self->can( "on_outgoing_empty" );
-
-      $on_outgoing_empty->( $self ) if $on_outgoing_empty;
+      $self->maybe_invoke_event( on_outgoing_empty => );
 
       $self->close_now if $self->{stream_closing};
    }
 }
 
-# Keep perl happy; keep Britain tidy
-1;
+=head1 UTILITY CONSTRUCTORS
 
-__END__
+=cut
+
+=head2 $stream = IO::Async::Stream->new_for_stdin
+
+=head2 $stream = IO::Async::Stream->new_for_stdout
+
+=head2 $stream = IO::Async::Stream->new_for_stdio
+
+Return a C<IO::Async::Stream> object preconfigured with the correct
+C<read_handle>, C<write_handle> or both.
+
+=cut
+
+sub new_for_stdin  { shift->new( read_handle  => \*STDIN, @_ ) }
+sub new_for_stdout { shift->new( write_handle => \*STDOUT, @_ ) }
+
+sub new_for_stdio { shift->new( read_handle => \*STDIN, write_handle => \*STDOUT, @_ ) }
 
 =head1 EXAMPLES
 
@@ -546,11 +683,10 @@ prints them to the program's C<STDOUT> stream.
  sub on_read
  {
     my $self = shift;
-    my ( $buffref, $handleclosed ) = @_;
+    my ( $buffref, $eof ) = @_;
 
-    if( $$buffref =~ s/^(.*\n)// ) {
+    while( $$buffref =~ s/^(.*\n)// ) {
        print "Received a line: $1";
-       return 1;
     }
 
     return 0;
@@ -563,6 +699,9 @@ available then C<0> is returned, to indicate it should not be tried again. If
 a line was successfully extracted, then C<1> is returned, to indicate it
 should try again in case more lines exist in the buffer.
 
+For implementing real network protocols that are based on lines of text it may
+be more appropriate to use a subclass of L<IO::Async::Protocol::LineStream>.
+
 =head2 Dynamic replacement of C<on_read()>
 
 Consider the following protocol (inspired by IMAP), which consists of
@@ -573,7 +712,7 @@ prefix.
  sub on_read
  {
     my $self = shift;
-    my ( $buffref, $handleclosed ) = @_;
+    my ( $buffref, $eof ) = @_;
 
     if( $$buffref =~ s/^DATA (\d+):(.*)\n// ) {
        my $length = $1;
@@ -581,7 +720,7 @@ prefix.
 
        return sub {
           my $self = shift;
-          my ( $buffref, $handleclosed ) = @_;
+          my ( $buffref, $eof ) = @_;
 
           return 0 unless length $$buffref >= $length;
 
@@ -626,3 +765,7 @@ L<IO::Handle> - Supply object methods for I/O handles
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
