@@ -8,7 +8,7 @@ package IO::Async::Test;
 use strict;
 use warnings;
 
-our $VERSION = '0.29';
+our $VERSION = '0.42';
 
 use Exporter 'import';
 our @EXPORT = qw(
@@ -17,8 +17,6 @@ our @EXPORT = qw(
    wait_for_stream
 );
 
-use IO::Async::Stream;
-
 =head1 NAME
 
 C<IO::Async::Test> - utility functions for use in test scripts
@@ -131,30 +129,40 @@ sub wait_for(&)
 
 =head2 wait_for_stream( $condfunc, $handle, $buffer )
 
-Set up an C<IO::Async::Stream> object around the given $handle. Data read from
-the stream will be appended into $buffer (which is NOT initialised when the
-function is entered, in case data remains from a previous call). The
-C<loop_once> method is then repeatedly called until the condition function
-callback returns true. After this, the temporary stream will be removed from
-the loop.
+As C<wait_for>, but will also watch the given IO handle for readability, and
+whenever it is readable will read bytes in from it into the given buffer. The
+buffer is NOT initialised when the function is entered, in case data remains
+from a previous call.
+
+C<$buffer> can also be a CODE reference, in which case it will be invoked
+being passed data read from the handle, whenever it is readable.
 
 =cut
 
 sub wait_for_stream(&$$)
 {
    my ( $cond, $handle, undef ) = @_;
-   my $varref = \$_[2]; # So that we can modify it from the on_read callback
+
+   my $on_read;
+   if( ref $_[2] eq "CODE" ) {
+      $on_read = $_[2];
+   }
+   else {
+      my $varref = \$_[2];
+      $on_read = sub { $$varref .= $_[0] };
+   }
 
    $loop->watch_io(
       handle => $handle,
       on_read_ready => sub {
-         my $ret = $handle->sysread( $$varref, 8192, length $$varref );
+         my $ret = $handle->sysread( my $buffer, 8192 );
          if( !defined $ret ) {
             die "Read failed on $handle - $!\n";
          }
          elsif( $ret == 0 ) {
             die "Read returned EOF on $handle\n";
          }
+         $on_read->( $buffer );
       }
    );
 
@@ -167,11 +175,10 @@ sub wait_for_stream(&$$)
    );
 }
 
-# Keep perl happy; keep Britain tidy
-1;
-
-__END__
-
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
