@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2011 -- leonerd@leonerd.org.uk
 
 package IO::Async::Timer;
 
@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Notifier );
 
-our $VERSION = '0.30';
+our $VERSION = '0.42';
 
 use Carp;
 
@@ -27,6 +27,10 @@ subclasses:
 
 =item *
 
+L<IO::Async::Timer::Absolute> - event callback at a fixed future time
+
+=item *
+
 L<IO::Async::Timer::Countdown> - event callback after a fixed delay
 
 =item *
@@ -149,7 +153,8 @@ sub start
 
 =head2 $timer->stop
 
-Stops the Timer if it is running.
+Stops the Timer if it is running. If it has not yet been added to the C<Loop>
+but there is a start pending, this will cancel it.
 
 =cut
 
@@ -157,6 +162,11 @@ sub stop
 {
    my $self = shift;
 
+   if( $self->{pending} ) {
+      delete $self->{pending};
+      return;
+   }
+
    my $loop = $self->get_loop or croak "Cannot stop a Timer that is not in a Loop";
 
    defined $self->{id} or return; # nothing to do but no error
@@ -166,11 +176,10 @@ sub stop
    undef $self->{id};
 }
 
-# Keep perl happy; keep Britain tidy
-1;
-
-__END__
-
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
