@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2010 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2011 -- leonerd@leonerd.org.uk
 
 package IO::Async::Timer::Countdown;
 
@@ -9,10 +9,9 @@ use strict;
 use warnings;
 use base qw( IO::Async::Timer );
 
-our $VERSION = '0.30';
+our $VERSION = '0.42';
 
 use Carp;
-use Scalar::Util qw( weaken );
 
 =head1 NAME
 
@@ -49,21 +48,19 @@ started again, when it will wait the same period then invoke the callback
 again. A timer that is currently running may be stopped or reset.
 
 For a C<Timer> object that repeatedly runs a callback at regular intervals,
-see instead L<IO::Async::Timer::Periodic>.
+see instead L<IO::Async::Timer::Periodic>. For a C<Timer> that invokes its
+callback at a fixed time in the future, see L<IO::Async::Timer::Absolute>.
 
-This object may be used in one of two ways; as an instance with CODE
-references as callbacks, or as a base class with overridden methods.
+=cut
 
-=over 4
+=head1 EVENTS
 
-=item Subclassing
+The following events are invoked, either using subclass methods or CODE
+references in parameters:
 
-If a subclass is built, then it can override the following methods to handle
-events:
+=head2 on_expire
 
- $self->on_expire()
-
-=back
+Invoked when the timer expires.
 
 =cut
 
@@ -75,10 +72,7 @@ The following named parameters may be passed to C<new> or C<configure>:
 
 =item on_expire => CODE
 
-CODE reference to callback to invoke when the timer expires. If not supplied,
-the subclass method will be called instead.
-
- $on_expire->( $self )
+CODE reference for the C<on_expire> event.
 
 =item delay => NUM
 
@@ -114,7 +108,7 @@ sub configure
       $self->{delay} = $delay;
    }
 
-   if( !$self->{on_expire} and !$self->can( 'on_expire' ) ) {
+   unless( $self->can_event( 'on_expire' ) ) {
       croak 'Expected either a on_expire callback or an ->on_expire method';
    }
 
@@ -124,20 +118,14 @@ sub configure
 sub _make_cb
 {
    my $self = shift;
-   weaken( my $weakself = $self );
 
-   if( $self->{on_expire} ) {
-      return sub {
-         undef $weakself->{id};
-         $weakself->{on_expire}->( $weakself );
-      };
-   }
-   else {
-      return sub {
-         undef $weakself->{id};
-         $weakself->on_expire;
-      };
-   }
+   return $self->_capture_weakself( sub {
+      my ( $self ) = @_;
+
+      undef $self->{id};
+
+      $self->invoke_event( "on_expire" );
+   } );
 }
 
 sub _make_enqueueargs
@@ -168,11 +156,6 @@ sub reset
    );
 }
 
-# Keep perl happy; keep Britain tidy
-1;
-
-__END__
-
 =head1 EXAMPLES
 
 =head2 Watchdog Timer
@@ -190,20 +173,22 @@ For example, to expire an accepted connection after 30 seconds of inactivity:
  on_accept => sub {
     my ( $newclient ) = @_;
 
-    my $stream;
-
     my $watchdog = IO::Async::Timer::Countdown->new(
        delay => 30,
 
-       on_expire => sub { $stream->close },
-    );
-    $stream->add_child( $watchdog );
+       on_expire => sub {
+          my $self = shift;
 
-    $stream = IO::Async::Stream->new(
+          my $stream = $self->parent;
+          $stream->close;
+       },
+    );
+
+    my $stream = IO::Async::Stream->new(
        handle => $newclient,
 
        on_read => sub {
-          my ( $self, $buffref, $closed ) = @_;
+          my ( $self, $buffref, $eof ) = @_;
           $watchdog->reset;
 
           ...
@@ -214,11 +199,52 @@ For example, to expire an accepted connection after 30 seconds of inactivity:
        },
     ) );
 
+    $stream->add_child( $watchdog );
     $watchdog->start;
 
     $loop->add( $watchdog );
  }
 
+Rather than setting up a lexical variable to store the Stream so that the
+Timer's C<on_expire> closure can call C<close> on it, the parent/child
+relationship between the two Notifier objects is used. At the time the Timer
+C<on_expire> closure is invoked, it will have been added as a child notifier
+of the Stream; this means the Timer's C<parent> method will return the Stream
+Notifier. This enables it to call C<close> without needing to capture a
+lexical variable, which would create a cyclic reference.
+
+=head2 Fixed-Delay Repeating Timer
+
+The C<on_expire> event fires a fixed delay after the C<start> method has begun
+the countdown. The C<start> method can be invoked again at some point during
+the C<on_expire> handling code, to create a timer that invokes its code
+regularly a fixed delay after the previous invocation has finished. This
+creates an arrangement similar to an L<IO::Async::Timer::Periodic>, except
+that it will wait until the previous invocation has indicated it is finished,
+before starting the countdown for the next call.
+
+ my $timer = IO::Async::Timer::Countdown->new(
+    delay => 60,
+
+    on_expire => sub {
+       my $self = shift;
+
+       start_some_operation(
+          on_complete => sub { $self->start },
+       );
+    },
+ );
+
+ $timer->start;
+ $loop->add( $timer );
+
+This example invokes the C<start_some_operation> function 60 seconds after the
+previous iteration has indicated it has finished.
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
