@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2010 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2011 -- leonerd@leonerd.org.uk
 
 package IO::Async::Timer::Periodic;
 
@@ -9,11 +9,9 @@ use strict;
 use warnings;
 use base qw( IO::Async::Timer );
 
-our $VERSION = '0.30';
+our $VERSION = '0.42';
 
 use Carp;
-use Scalar::Util qw( weaken );
-use Time::HiRes qw( time );
 
 =head1 NAME
 
@@ -48,21 +46,21 @@ to execute, but runs at regular intervals beginning at the time the timer was
 started, then adding each interval thereafter.
 
 For a C<Timer> object that only runs a callback once, after a given delay, see
-instead L<IO::Async::Timer::Countdown>.
+instead L<IO::Async::Timer::Countdown>. A Countdown timer can also be used to
+create repeating events that fire at a fixed delay after the previous event
+has finished processing. See als the examples in
+C<IO::Async::Timer::Countdown>.
 
-This object may be used in one of two ways; as an instance with CODE
-references as callbacks, or as a base class with overridden methods.
+=cut
 
-=over 4
+=head1 EVENTS
 
-=item Subclassing
+The following events are invoked, either using subclass methods or CODE
+references in parameters:
 
-If a subclass is built, then it can override the following methods to handle
-events:
+=head2 on_tick
 
- $self->on_tick()
-
-=back
+Invoked on each interval of the timer.
 
 =cut
 
@@ -74,16 +72,24 @@ The following named parameters may be passed to C<new> or C<configure>:
 
 =item on_tick => CODE
 
-CODE reference to callback to invoke each interval. If not supplied, the
-subclass method will be called instead.
-
- $on_tick->( $self )
+CODE reference for the C<on_tick> event.
 
 =item interval => NUM
 
 The interval in seconds between invocations of the callback or method. Cannot
 be changed if the timer is running.
 
+=item first_interval => NUM
+
+Optional. If defined, the interval in seconds after calling the C<start>
+method before the first invocation of the callback or method. Thereafter, the
+regular C<interval> will be used. If not supplied, the first interval will be
+the same as the others.
+
+Even if this value is zero, the first invocation will be made asynchronously,
+by the containing C<Loop> object, and not synchronously by the C<start> method
+itself.
+
 =back
 
 Once constructed, the timer object will need to be added to the C<Loop> before
@@ -113,23 +119,43 @@ sub configure
       $self->{interval} = $interval;
    }
 
-   if( !$self->{on_tick} and !$self->can( 'on_tick' ) ) {
+   if( exists $params{first_interval} ) {
+      $self->is_running and croak "Cannot configure 'first_interval' of a running timer\n";
+
+      my $first_interval = delete $params{first_interval};
+      $first_interval >= 0 or croak "Expected a 'first_interval' as a non-negative number";
+
+      $self->{first_interval} = $first_interval;
+   }
+
+   unless( $self->can_event( 'on_tick' ) ) {
       croak 'Expected either a on_tick callback or an ->on_tick method';
    }
 
    $self->SUPER::configure( %params );
 }
 
+sub _next_interval
+{
+   my $self = shift;
+   return $self->{first_interval} if defined $self->{first_interval};
+   return $self->{interval};
+}
+
 sub start
 {
    my $self = shift;
 
-   my $now = time;
-   if( !defined $self->{next_time} ) {
-      $self->{next_time} = time + $self->{interval};
-   }
-   else {
-      $self->{next_time} += $self->{interval};
+   # Only actually define a time if we've got a loop; otherwise it'll just
+   # become start-pending. We'll calculate it properly when it gets added to
+   # the Loop
+   if( my $loop = $self->get_loop ) {
+      if( !defined $self->{next_time} ) {
+         $self->{next_time} = $loop->time + $self->_next_interval;
+      }
+      else {
+         $self->{next_time} += $self->_next_interval;
+      }
    }
 
    $self->SUPER::start;
@@ -146,22 +172,17 @@ sub stop
 sub _make_cb
 {
    my $self = shift;
-   weaken( my $weakself = $self );
 
-   if( $self->{on_tick} ) {
-      return sub {
-         undef $weakself->{id};
-         $weakself->start;
-         $weakself->{on_tick}->( $weakself );
-      };
-   }
-   else {
-      return sub {
-         undef $weakself->{id};
-         $weakself->start;
-         $weakself->on_tick;
-      };
-   }
+   return $self->_capture_weakself( sub {
+      my $self = shift;
+
+      undef $self->{first_interval};
+
+      undef $self->{id};
+      $self->start;
+
+      $self->invoke_event( on_tick => );
+   } );
 }
 
 sub _make_enqueueargs
@@ -171,11 +192,10 @@ sub _make_enqueueargs
    return time => $self->{next_time};
 }
 
-# Keep perl happy; keep Britain tidy
-1;
-
-__END__
-
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
